import { Doctor } from "../home/Doctor";
import { Version } from "../home/Version";

export const CLI = () => {
    return (
        <>
            <Doctor />
            <Version />
        </>
    );
};
